/* $Id: SrvCapacityPlanner.sql,v 1.29 2011/04/13 22:36:25 bdighe Exp $*/
/* $Copyright: Copyright (c) 2011 Symantec Corporation.
 * All rights reserved.
 *
 * THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
 * SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
 * WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
 *
 * The Licensed Software and Documentation are deemed to be commercial
 * computer software as defined in FAR 12.212 and subject to restricted
 * rights as defined in FAR Section 52.227-19 "Commercial Computer
 * Software - Restricted Rights" and DFARS 227.7202, "Rights in
 * Commercial Computer Software or Commercial Computer Software
 * Documentation", as applicable, and any successor regulations. Any use,
 * modification, reproduction release, performance, display or disclosure
 * of the Licensed Software and Documentation by the U.S. Government
 * shall be solely in accordance with the terms of this Agreement.  $ */

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_UI_SERVER_POOL' and table_type='VIEW') then
   drop view SRVCAP_VIEW_UI_SERVER_POOL
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_UI_SERVER_POOL as
select 
    pool.GRP_ID as GRP_ID,
    be.GRP_NAME as GRP_NAME,
   	pool.CPUS_PER_SERVER as CPUS_PER_SERVER, 
   	pool.MEMORY_PER_SERVER as MEMORY_PER_SERVER,
   	pool.FAULT_TOLERANCE as FAULT_TOLERANCE
from  
    SRVCAP_VIEW_SERVER_POOL_R pool
    inner join P_SF_GENERIC_VIEW_GROUP be on be.GRP_ID=pool.GRP_ID

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_UI_SERVER_POOL to HABADM;
grant SELECT                      on SRVCAP_VIEW_UI_SERVER_POOL to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_UI_SERVER_POOL to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_UI_SERVER_POOL to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_SERVER_POOL_HOSTS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_SERVER_POOL_HOSTS
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_SERVER_POOL_HOSTS as
SELECT 
    DISTINCT pool.GRP_ID as GRP_ID, 
    pool.GRP_NAME as GRP_NAME,
   	pool.CPUS_PER_SERVER as CPUS_PER_SERVER, 
   	pool.MEMORY_PER_SERVER as MEMORY_PER_SERVER,
   	pool.FAULT_TOLERANCE as FAULT_TOLERANCE,
    static.OBJECT_ID as HOST_ID,
    vserver.SUB_TYPE as HOST_TYPE,
    vserver.CPUS as HOST_CPUS,
    vserver.MEMORY as HOST_MEMORY,
    hostToClus.HC_ID as HC_ID
FROM 
	SRVCAP_VIEW_UI_SERVER_POOL pool 
INNER JOIN SF_VIEW_STATIC_VIEW_HIER static on static.OBJECT_TYPE='vrts_host' and static.grp_id=pool.grp_id
LEFT OUTER JOIN SF_VIEW_HOST_TO_HC_R hostToClus on hostToClus.TYPE is NULL and hostToClus.HOST_ID=static.OBJECT_ID
LEFT OUTER JOIN SF_VIEW_HOST_CONTAINER_R vserver on vserver.ID=static.OBJECT_ID

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_SERVER_POOL_HOSTS to HABADM;
grant SELECT                      on SRVCAP_VIEW_SERVER_POOL_HOSTS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_SERVER_POOL_HOSTS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_SERVER_POOL_HOSTS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_BE_HOSTS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_BE_HOSTS
end if;

CREATE VIEW SRVCAP_VIEW_BE_HOSTS as
SELECT 
    DISTINCT static.GRP_ID as GRP_ID, 
    static.OBJECT_ID as HOST_ID,
    vserver.SUB_TYPE as HOST_TYPE,
    vserver.CPUS as HOST_CPUS,
    vserver.MEMORY as HOST_MEMORY,
    hostToClus.HC_ID as HC_ID
FROM 
	SF_VIEW_STATIC_VIEW_HIER static
LEFT OUTER JOIN SF_VIEW_HOST_TO_HC_R hostToClus on hostToClus.TYPE is NULL and hostToClus.HOST_ID=static.OBJECT_ID
LEFT OUTER JOIN SF_VIEW_HOST_CONTAINER_R vserver on vserver.ID=static.OBJECT_ID
WHERE Object_type='vrts_host'

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_HOSTS to HABADM;
grant SELECT                      on SRVCAP_VIEW_BE_HOSTS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_HOSTS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_HOSTS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_BE_CLUSTERS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_BE_CLUSTERS
end if;

CREATE VIEW SRVCAP_VIEW_BE_CLUSTERS as
SELECT 
    DISTINCT static.GRP_ID as GRP_ID, 
    hc.ID as HC_ID,
    hc.NAME as HC_NAME,
    hc.SUB_TYPE as HC_SUBTYPE,
	hc.HA_TYPE as HC_HATYPE,
    hc.NHOSTS as NHOSTS
    
FROM 
	SF_VIEW_STATIC_VIEW_HIER static
LEFT OUTER JOIN SF_VIEW_HOST_TO_HC_R hostToClus on hostToClus.TYPE is NULL and hostToClus.HOST_ID=static.OBJECT_ID
LEFT OUTER JOIN SF_VIEW_HOST_CONTAINER_R hc on hc.ID=hostToClus.HC_ID

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_CLUSTERS to HABADM;
grant SELECT                      on SRVCAP_VIEW_BE_CLUSTERS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_CLUSTERS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_BE_CLUSTERS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_VM_TO_VSERVER' and table_type='VIEW') then
   drop view SRVCAP_VIEW_VM_TO_VSERVER
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_VM_TO_VSERVER as
SELECT 
	host.name as VSERVER_NAME, 
	hc.ID as VSERVER_ID, 
	vm.allocated_memory as VM_ALLOCATED_MEM, 
	vm.vm_name as  VM_NAME, 
	vm.vprocessors as VM_VPROCESSORS,
	vm.is_control as IS_CONTROL
FROM VM_VIEW_INFO_R vm
INNER JOIN VM_VIEW_HC_REL_R hhc on hhc.VM_ID=vm.id
INNER JOIN SF_VIEW_HOST_CONTAINER_R hc on hc.ID=hhc.HC_ID
INNER JOIN SF_VIEW_HOST_R host on host.ID=hc.ID

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_VM_TO_VSERVER to HABADM;
grant SELECT                      on SRVCAP_VIEW_VM_TO_VSERVER to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_VM_TO_VSERVER to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_VM_TO_VSERVER to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_ONLINE_LDOMS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_ONLINE_LDOMS
end if;

CREATE VIEW SRVCAP_VIEW_ONLINE_LDOMS as

select 
    hc.ID as HC_ID,
	hc.NAME as HC_NAME,
	hc.sub_type as HC_SUBTYPE,
	hc.ha_type as HC_HATYPE,
	hc.NHOSTS as HC_NHOSTS,

	sg.id AS SG_ID,  
    sg.name as SG_NAME ,
	sg.scope as SG_SCOPE,
	sg.frozen as SG_FROZEN, 
	sg.state as SG_STATE , 

    res.ID as RES_ID,
    res.STORAGE_TYPE as STORAGE_TYPE,
    res.STATE as STATE,

    hr.STATE as STATE_ON_HOST,
    hr.HOST_ID as HOST_ID,
    hr.STORAGE_ID as STORAGE_ID,
    
    uhost.VM_NAME as VM_NAME,
    uhost.VM_VPROCESSORS as VM_VPROCESSORS,
    uhost.VM_ALLOCATED_MEM as VM_ALLOCATED_MEM,
    uhost.VSERVER_ID as VSERVER_ID,
    uhost.VSERVER_NAME as VSERVER_NAME,
    
    NULL as NOTES,
    NULL as USER_ID,
    NULL as LAST_UPDATE_TIME
	
from  
    VCS_VIEW_REL_HOST_RES_R hr
    inner join VCS_VIEW_RESOURCE_R  res on res.ID=hr.RES_ID
    inner join VCS_VIEW_SERVICEGROUP_R sg on sg.ID=res.SG_ID
    inner join SF_VIEW_HOST_CONTAINER hc on hc.HA_TYPE='VCS' and hc.ID=sg.HC_ID 
    left outer join SRVCAP_VIEW_VM_TO_VSERVER uHost on uHost.VM_NAME=hr.STORAGE_ID and uHost.VSERVER_ID=hr.HOST_ID
where 
    STORAGE_TYPE='LDom' AND STATE=1 AND STATE_ON_HOST=1
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_ONLINE_LDOMS to HABADM;
grant SELECT                      on SRVCAP_VIEW_ONLINE_LDOMS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_ONLINE_LDOMS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_ONLINE_LDOMS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_OFFLINE_LDOMS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_OFFLINE_LDOMS
end if;

CREATE VIEW SRVCAP_VIEW_OFFLINE_LDOMS as
select 
distinct     
    hc.ID as HC_ID,
	hc.NAME as HC_NAME,
	hc.sub_type as HC_SUBTYPE,
	hc.ha_type as HC_HATYPE,
	hc.NHOSTS as HC_NHOSTS,

	sg.id AS SG_ID,  
    sg.name as SG_NAME ,
	sg.scope as SG_SCOPE,
	sg.frozen as SG_FROZEN, 
	sg.state as SG_STATE , 

    res.ID as RES_ID,
    res.STORAGE_TYPE as STORAGE_TYPE,
    0 as STATE,

    hr.STATE as STATE_ON_HOST,
    NULL as HOST_ID,
    hr.STORAGE_ID as STORAGE_ID,
    
    NULL as VM_NAME,
    CASE WHEN resrv.CPUS is null THEN 0  ELSE resrv.CPUS END AS VM_VPROCESSORS,
    CASE WHEN resrv.MEMORY is null THEN 0  ELSE resrv.MEMORY END AS VM_ALLOCATED_MEM,
    NULL as VSERVER_ID,
    NULL as VSERVER_NAME,
    
    NULL as NOTES,
    NULL as USER_ID,
    NULL as LAST_UPDATE_TIME
	
from  
    VCS_VIEW_REL_HOST_RES_R hr
    inner join VCS_VIEW_RESOURCE_R  res on res.ID=hr.RES_ID
    inner join VCS_VIEW_SERVICEGROUP_R sg on sg.ID=res.SG_ID
    inner join SF_VIEW_HOST_CONTAINER hc on hc.HA_TYPE='VCS' and hc.ID=sg.HC_ID
    left outer join SRVCAP_VIEW_RESERVED_VMS_R resrv on resrv.HC_ID=hc.ID and resrv.VM_NAME=hr.STORAGE_ID
where 
    STORAGE_TYPE='LDom' AND res.STATE!=1
   
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_OFFLINE_LDOMS to HABADM;
grant SELECT                      on SRVCAP_VIEW_OFFLINE_LDOMS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_OFFLINE_LDOMS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_OFFLINE_LDOMS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS
end if;

CREATE VIEW SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS as

select 
    hc.ID as HC_ID,
	hc.NAME as HC_NAME,
	hc.sub_type as HC_SUBTYPE,
	hc.ha_type as HC_HATYPE,
	hc.NHOSTS as HC_NHOSTS,

	NULL AS SG_ID,  
    NULL as SG_NAME ,
	NULL as SG_SCOPE,
	NULL as SG_FROZEN, 
	NULL as SG_STATE , 

    NULL as RES_ID,
    NULL as STORAGE_TYPE,
    -1 as STATE,

    -1 as STATE_ON_HOST,
    null as HOST_ID,
    reserv.VM_NAME as STORAGE_ID,
    
    reserv.VM_NAME as VM_NAME,
    reserv.CPUS as VM_VPROCESSORS,
    reserv.MEMORY as VM_ALLOCATED_MEM,
    NULL as VSERVER_ID,
    NULL as VSERVER_NAME,
    
    reserv.NOTES as NOTES,
    reserv.USER_ID as RESERVED_BY,
    reserv.LAST_UPDATE_TIME as LAST_UPDATE_TIME
from
	SRVCAP_VIEW_RESERVED_VMS_R reserv
	left outer join SF_VIEW_HOST_CONTAINER hc on hc.ID=reserv.HC_ID
	left outer join SRVCAP_VIEW_ONLINE_LDOMS online on online.HC_ID=reserv.HC_ID and online.STORAGE_ID=reserv.VM_NAME
	left outer join SRVCAP_VIEW_OFFLINE_LDOMS offline on offline.HC_ID=reserv.HC_ID and offline.STORAGE_ID=reserv.VM_NAME
where 
    online.HC_ID is NULL AND offline.HC_ID is NULL

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS to HABADM;
grant SELECT                      on SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_CLUSTERED_LDOMS' and table_type='VIEW') then
   drop view SRVCAP_VIEW_CLUSTERED_LDOMS
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_CLUSTERED_LDOMS as

select * from SRVCAP_VIEW_ONLINE_LDOMS
union all
select * from SRVCAP_VIEW_OFFLINE_LDOMS
union all
select * from SRVCAP_VIEW_RESERVED_NOT_CONFIGURED_LDOMS

grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS to HABADM;
grant SELECT                      on SRVCAP_VIEW_CLUSTERED_LDOMS to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL' and table_type='VIEW') then
   drop view SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL as
SELECT
	distinct
	pool.GRP_ID as GRP_ID,
	pool.GRP_NAME as GRP_NAME,
   	pool.CPUS_PER_SERVER as CPUS_PER_SERVER, 
   	pool.MEMORY_PER_SERVER as MEMORY_PER_SERVER,
   	pool.FAULT_TOLERANCE as FAULT_TOLERANCE,
	
	ldomres.*
FROM
    SRVCAP_VIEW_SERVER_POOL_HOSTS pool
    left outer join SRVCAP_VIEW_CLUSTERED_LDOMS ldomres on pool.HC_ID=ldomres.HC_ID
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL to HABADM;
grant SELECT                      on SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CLUSTERED_LDOMS_TO_POOL to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_CTRLDMN_CLUSTER' and table_type='VIEW') then
   drop view SRVCAP_VIEW_CTRLDMN_CLUSTER
end if;

CREATE VIEW SRVCAP_VIEW_CTRLDMN_CLUSTER as
SELECT 
	hostToClus.HC_ID as HC_ID,
	max(vm.VPROCESSORS) as MAX_VPROC,
	max(vm.ALLOCATED_MEMORY) as MAX_MEMORY
FROM 
	VM_VIEW_INFO_R vm
	INNER JOIN VM_VIEW_HC_REL_R vmToHost on vmToHost.VM_ID=vm.ID
	INNER JOIN SF_VIEW_HOST_R  host on host.ID=vmToHost.HC_ID
	INNER JOIN SF_VIEW_HOST_TO_HC hostToClus on hostToClus.HOST_ID=vmToHost.HC_ID
WHERE vm.is_control=1
GROUP BY HC_ID;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER to HABADM;
grant SELECT                      on SRVCAP_VIEW_CTRLDMN_CLUSTER to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER to HABGUI;

if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL' and table_type='VIEW') then
   drop view SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL
end if;

CREATE VIEW SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL as
SELECT 
	beToClus.GRP_ID as GRP_ID,
	vm.*
FROM 
	SRVCAP_VIEW_CTRLDMN_CLUSTER vm
	INNER JOIN SRVCAP_VIEW_BE_CLUSTERS beToClus on beToClus.HC_ID=vm.HC_ID
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL to HABADM;
grant SELECT                      on SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SRVCAP_VIEW_CTRLDMN_CLUSTER_POOL to HABGUI;


if exists(select 1 from sys.systable where table_name='SRVCAP_VIEW_CLUSTER_CONSUMPTION' and table_type='VIEW') then
   drop view SRVCAP_VIEW_CLUSTER_CONSUMPTION
end if;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_CLUSTER_CONSUMPTION AS
SELECT 
	HC.HC_ID as HC_ID,
	HC.HC_NAME as HC_NAME,
	HC.HC_SUBTYPE as HC_SUBTYPE,
	HC.HC_HATYPE as HC_HATYPE,
	HC.NHOSTS as HC_NHOSTS,
	COALESCE(ctrlDmn.MAX_VPROC, 0) as CTRL_VPROC,
	COALESCE(ctrlDmn.MAX_MEMORY, 0) as CTRL_MEMORY,
	COALESCE(SUM(SUM_ONLINE_CPU),0) AS CLUS_SUM_ONLINE_CPU,
	COALESCE(SUM(SUM_OFFLINE_CPU),0) AS CLUS_SUM_OFFLINE_CPU,
	COALESCE(SUM(SUM_RESERVE_CPU),0) AS CLUS_SUM_RESERVE_CPU,
	COALESCE(SUM(SUM_ONLINE_MEM),0) AS CLUS_SUM_ONLINE_MEM,
	COALESCE(SUM(SUM_OFFLINE_MEM),0) AS CLUS_SUM_OFFLINE_MEM,
	COALESCE(SUM(SUM_RESERVE_MEM),0) AS CLUS_SUM_RESERVE_MEM,
	COALESCE(SUM(ONLINE_CNT),0) AS CLUS_ONLINE_LDOM_CNT,
	COALESCE(SUM(OFFLINE_CNT),0) AS CLUS_OFFLINE_LDOM_CNT,
	COALESCE(SUM(RESRV_CNT),0) AS CLUS_RESRV_LDOM_CNT
    FROM
        SRVCAP_VIEW_BE_CLUSTERS HC
        INNER JOIN SRVCAP_VIEW_UI_SERVER_POOL POOL on POOL.GRP_ID=HC.GRP_ID
 	    LEFT OUTER JOIN SRVCAP_VIEW_CTRLDMN_CLUSTER ctrlDmn on ctrlDmn.HC_ID=HC.HC_ID
    	LEFT OUTER JOIN
    	(
    	    SELECT 
    	    HC_ID as HC_ID, 
    	    CASE WHEN STATE=1 THEN COALESCE(SUM_CPU,0)  ELSE 0 END AS SUM_ONLINE_CPU,
    	    CASE WHEN STATE=0 THEN COALESCE(SUM_CPU,0)  ELSE 0 END AS SUM_OFFLINE_CPU,
    	    CASE WHEN STATE=-1 THEN COALESCE(SUM_CPU,0)  ELSE 0 END AS SUM_RESERVE_CPU,
    	    CASE WHEN STATE=1 THEN COALESCE(SUM_MEM,0)  ELSE 0 END AS SUM_ONLINE_MEM,
    	    CASE WHEN STATE=0 THEN COALESCE(SUM_MEM,0)  ELSE 0 END AS SUM_OFFLINE_MEM,
    	    CASE WHEN STATE=-1 THEN COALESCE(SUM_MEM,0)  ELSE 0 END AS SUM_RESERVE_MEM,
    	    CASE WHEN STATE=1 THEN COALESCE(CNT_STATE,0)  ELSE 0 END AS ONLINE_CNT,
    	    CASE WHEN STATE=0 THEN COALESCE(CNT_STATE,0) ELSE 0 END AS OFFLINE_CNT,
    	    CASE WHEN STATE=-1 THEN COALESCE(CNT_STATE,0)  ELSE 0 END AS RESRV_CNT
    	    FROM
    	    (
    	        SELECT 
    		        ldom.HC_ID as HC_ID, 
    		        SUM(VM_VPROCESSORS) AS SUM_CPU, 
    		        SUM(VM_ALLOCATED_MEM) AS SUM_MEM, 
    		        STATE, 
    		        COUNT(STATE) AS CNT_STATE 
    	        FROM  SRVCAP_VIEW_CLUSTERED_LDOMS ldom
    	        GROUP BY HC_ID, STATE
    	    ) CLUS1
    	) CLUS2 on CLUS2.HC_ID=HC.HC_ID
	GROUP BY HC_ID, HC_NAME, HC_SUBTYPE, HC_HATYPE, HC_NHOSTS, CTRL_VPROC, CTRL_MEMORY
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION TO HABADM;
GRANT SELECT                      ON SRVCAP_VIEW_CLUSTER_CONSUMPTION TO HABGUEST;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION TO HABDBSYNC;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION TO HABGUI;


IF EXISTS(SELECT 1 FROM SYS.SYSTABLE WHERE TABLE_NAME='SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL' AND TABLE_TYPE='VIEW') THEN
   DROP VIEW SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL
END IF;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL AS
SELECT 
DISTINCT
	POOL.GRP_ID AS GRP_ID,
	POOL.GRP_NAME AS GRP_NAME,
	
	SUMS.*,
    (CPUS_PER_SERVER * (HC_NHOSTS - FAULT_TOLERANCE)) - (CLUS_SUM_ONLINE_CPU + CLUS_SUM_OFFLINE_CPU + CLUS_SUM_RESERVE_CPU +  (HC_NHOSTS * CTRL_VPROC)) AS AVAIL_CPU,
    (MEMORY_PER_SERVER * (HC_NHOSTS - FAULT_TOLERANCE)) - (CLUS_SUM_ONLINE_MEM + CLUS_SUM_OFFLINE_MEM + CLUS_SUM_RESERVE_MEM + (HC_NHOSTS * CTRL_MEMORY)) AS AVAIL_MEM,
    POOL.CPUS_PER_SERVER AS CPUS_PER_SERVER, 
	POOL.MEMORY_PER_SERVER AS MEMORY_PER_SERVER,
	POOL.FAULT_TOLERANCE AS FAULT_TOLERANCE

FROM
    SRVCAP_VIEW_SERVER_POOL_HOSTS POOL
    LEFT OUTER JOIN SRVCAP_VIEW_CLUSTER_CONSUMPTION SUMS ON POOL.HC_ID=SUMS.HC_ID
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL TO HABADM;
GRANT SELECT                      ON SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL TO HABGUEST;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL TO HABDBSYNC;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL TO HABGUI;

IF EXISTS(SELECT 1 FROM SYS.SYSTABLE WHERE TABLE_NAME='SRVCAP_VIEW_POOL_CONSUMPTION' AND TABLE_TYPE='VIEW') THEN
   DROP VIEW SRVCAP_VIEW_POOL_CONSUMPTION
END IF;

CREATE VIEW HABDBSYNC.SRVCAP_VIEW_POOL_CONSUMPTION AS
SELECT 
DISTINCT
	POOL.GRP_ID AS GRP_ID,
	POOL.GRP_NAME AS GRP_NAME,
    POOL.CPUS_PER_SERVER AS CPUS_PER_SERVER, 
	POOL.MEMORY_PER_SERVER AS MEMORY_PER_SERVER,
	POOL.FAULT_TOLERANCE AS FAULT_TOLERANCE,
	COUNT(POOL.HC_ID) AS POOL_CLUS_COUNT,
	SUM(POOL.HC_NHOSTS) AS POOL_HOST_COUNT,
	SUM(POOL.AVAIL_CPU) AS POOL_SUM_AVAIL_CPU,
	SUM(POOL.AVAIL_MEM) AS POOL_SUM_AVAIL_MEM,
	SUM(POOL.CLUS_SUM_ONLINE_CPU) AS POOL_SUM_ONLINE_CPU,
	SUM(POOL.CLUS_SUM_OFFLINE_CPU) AS POOL_SUM_OFFLINE_CPU,
	SUM(POOL.CLUS_SUM_RESERVE_CPU) AS POOL_SUM_RESERVE_CPU,
	SUM(POOL.CLUS_SUM_ONLINE_MEM) AS POOL_SUM_ONLINE_MEM,
	SUM(POOL.CLUS_SUM_OFFLINE_MEM) AS POOL_SUM_OFFLINE_MEM,
	SUM(POOL.CLUS_SUM_RESERVE_MEM) AS POOL_SUM_RESERVE_MEM,
	SUM(POOL.CLUS_ONLINE_LDOM_CNT) AS POOL_ONLINE_LDOM_CNT,
	SUM(POOL.CLUS_OFFLINE_LDOM_CNT) AS POOL_OFFLINE_LDOM_CNT,
	SUM(POOL.CLUS_RESRV_LDOM_CNT) AS POOL_RESRV_LDOM_CNT
FROM
    SRVCAP_VIEW_CLUSTER_CONSUMPTION_TO_POOL POOL
GROUP BY 
	GRP_ID,GRP_NAME,CPUS_PER_SERVER,MEMORY_PER_SERVER,FAULT_TOLERANCE
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CONSUMPTION TO HABADM;
GRANT SELECT                      ON SRVCAP_VIEW_POOL_CONSUMPTION TO HABGUEST;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CONSUMPTION TO HABDBSYNC;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CONSUMPTION TO HABGUI;

IF EXISTS(SELECT 1 FROM SYS.SYSTABLE WHERE TABLE_NAME='SRVCAP_VIEW_POOL_CAPACITY' AND TABLE_TYPE='VIEW') THEN
   DROP VIEW SRVCAP_VIEW_POOL_CAPACITY
END IF;

CREATE VIEW SRVCAP_VIEW_POOL_CAPACITY AS
SELECT 
DISTINCT
	POOL.GRP_ID AS GRP_ID,
	POOL.GRP_NAME AS GRP_NAME,
	COUNT(HC_ID) as CLUS_COUNT,
	SUM(NHOSTS) as HOST_COUNT,
	CPUS_PER_SERVER * (HOST_COUNT - (CLUS_COUNT * FAULT_TOLERANCE)) as CPU_CAPACITY, 
	MEMORY_PER_SERVER * (HOST_COUNT - (CLUS_COUNT * FAULT_TOLERANCE)) as MEM_CAPACITY 
FROM
    SRVCAP_VIEW_UI_SERVER_POOL POOL
    INNER JOIN SRVCAP_VIEW_BE_CLUSTERS clus on clus.GRP_ID=POOL.GRP_ID
GROUP BY GRP_ID, GRP_NAME, CPUS_PER_SERVER, MEMORY_PER_SERVER, FAULT_TOLERANCE
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CAPACITY TO HABADM;
GRANT SELECT                      ON SRVCAP_VIEW_POOL_CAPACITY TO HABGUEST;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CAPACITY TO HABDBSYNC;
GRANT DELETE,INSERT,UPDATE,SELECT ON SRVCAP_VIEW_POOL_CAPACITY TO HABGUI;

DECLARE LOCAL TEMPORARY TABLE POOL_FIDS
(FID varchar(128), MSG varchar(1024), OBJ_TYPE varchar(100))
NOT TRANSACTIONAL;

insert into POOL_FIDS values ('FID_POOL_MEMORY_MISMATCH','[Capacity Planner] Memory non-compliant with organizational entity definition','vrts_host');
insert into POOL_FIDS values ('FID_POOL_CPU_MISMATCH','[Capacity Planner] Number of processors non-compliant with organizational entity definition','vrts_host');
insert into POOL_FIDS values ('FID_POOL_FT_ERROR','[Capacity Planner] Spare servers setting cannot be honored for cluster in the organizational entity', 'vrts_host');
insert into POOL_FIDS values ('FID_POOL_CLUS_HOST_MISSING_ERROR','[Capacity Planner] One or more hosts belonging to the cluster %1 are not added to the organizational entity','vrts_cluster');
insert into POOL_FIDS values ('FID_POOL_CLUS_HOST_NOCLUS_ERROR','[Capacity Planner] Host %1 does not belong to any cluster','vrts_cluster');

insert into P_FAULT_DEFINITION (OBJ_TYPE, FID,TYPE,BASE, SYS_DEFINED_CONDITION,SCOPE,UPDATE_TIME) 
on existing update 
select POOL_FIDS.OBJ_TYPE,POOL_FIDS.FID, 1, 1, 1, 'I',CURRENT TIME from POOL_FIDS;     


insert into P_UI_RESOURCE (ID) on existing update
select FID from POOL_FIDS; 

insert into P_UI_RES_BUNDLE (ID,LOCALE_ID,DISPLAY) on existing update
select FID,'en_US', MSG from POOL_FIDS;


if exists(select 1 from sys.systable where table_name='SF_VIEW_UI_CAPACITY_FAULTS_DATA' and table_type='VIEW') then
   drop view SF_VIEW_UI_CAPACITY_FAULTS_DATA
end if;

CREATE VIEW HABDBSYNC.SF_VIEW_UI_CAPACITY_FAULTS_DATA as

select 
	HTHC.HC_ID, 
	S.GRP_ID, 
	OBJECT_ID, 
	NAME, 
	FAULT_TOLERANCE, 
	MEMORY_PER_SERVER, 
	MEMORY, 
	CPUS_PER_SERVER, 
	CPUS,
  	hthc3.clus_hosts as cluster_node_count, 
    hthc3.count as be_cluster_node_count,
    cluster_name
from SRVCAP_SERVER_POOL P

join SF_VIEW_STATIC_VIEW_HIER S on S.GRP_ID=P.GRP_ID and OBJECT_TYPE='vrts_host'
join SF_VIEW_HOST_CONTAINER_R HC on HC.ID=S.OBJECT_ID 
left outer join P_SF_HOST_TO_HC HTHC on HTHC.HOST_ID=S.OBJECT_ID

left outer join 
(
select HC_ID, 
GRP_ID, 
HC1.NAME as cluster_name, 
NHOSTS as clus_hosts, 
count(*) as count 
from P_SF_HOST_TO_HC HTHC 
join SF_VIEW_STATIC_VIEW_HIER S1 on  HTHC.HOST_ID=S1.OBJECT_ID 
join P_SF_HOST_CONTAINER HC1 on HC1.ID=HTHC.HC_ID and HC1.TYPE='CLUSTER'

group by HC_ID,GRP_ID, NAME, clus_hosts) hthc3 on hthc3.GRP_ID=P.GRP_ID and hthc3.HC_ID=hthc.HC_ID

grant DELETE,INSERT,UPDATE,SELECT on SF_VIEW_UI_CAPACITY_FAULTS_DATA to HABADM;
grant SELECT                      on SF_VIEW_UI_CAPACITY_FAULTS_DATA to HABGUEST;
grant DELETE,INSERT,UPDATE,SELECT on SF_VIEW_UI_CAPACITY_FAULTS_DATA to HABDBSYNC;
grant DELETE,INSERT,UPDATE,SELECT on SF_VIEW_UI_CAPACITY_FAULTS_DATA to HABGUI;


IF EXISTS ( SELECT 1 FROM sys.sysprocedure WHERE proc_name = 'SP_create_capacity_faults')
then
    drop procedure habdbsync.SP_create_capacity_faults 
END IF;

CREATE PROCEDURE SP_create_capacity_faults
as
declare 
	@hcId varchar(1024),
    @beId varchar(100), 
    @objId varchar(100), 
    @name varchar(100), 
    @ft bigint, 
    @hostcount bigint,
    @memoryPerServer unsigned bigint, 
    @memory unsigned bigint, 
    @cps bigint, 
    @cpu bigint,
    @clusterHostCount bigint,
    @clusterHostCountBE bigint,
    @clusterName varchar(100)
declare c1 cursor for 
select * from SF_VIEW_UI_CAPACITY_FAULTS_DATA
open c1

delete from P_FAULT_OBJ where FID='FID_POOL_CLUS_HOST_MISSING_ERROR' or FID='FID_POOL_CLUS_HOST_NOCLUS_ERROR' or FID='FID_POOL_MEMORY_MISMATCH' or FID='FID_POOL_CPU_MISMATCH' or FID = 'FID_POOL_FT_ERROR'

fetch first c1 into @hcId,@beId, @objId, @name, @ft, @memoryPerServer, @memory, @cps, @cpu,@clusterHostCount,@clusterHostCountBE,@clusterName

while(@@sqlstatus = 0)
	begin
    
    IF (@memoryPerServer != @memory )
    begin
      insert into P_FAULT_OBJ (TIME_RECEIVED,OBJ_TYPE,OBJ_ID,FID,SOURCE_TYPE,SOURCE_ID, OBJ_NAME, SOURCE_NAME) on existing update VALUES (CURRENT TIME,'vrts_host',@objId,'FID_POOL_MEMORY_MISMATCH', 'vrts_host',@objId, @name, @name)
    end

    IF (@cps != @cpu)
    begin
      insert into P_FAULT_OBJ (TIME_RECEIVED,OBJ_TYPE,OBJ_ID,FID,SOURCE_TYPE,SOURCE_ID, OBJ_NAME, SOURCE_NAME) on existing update VALUES (CURRENT TIME,'vrts_host',@objId,'FID_POOL_CPU_MISMATCH', 'vrts_host',@objId, @name, @name)
    end

    IF (@ft >= @hostcount )
    begin
      insert into P_FAULT_OBJ (TIME_RECEIVED,OBJ_TYPE,OBJ_ID,FID,SOURCE_TYPE,SOURCE_ID, OBJ_NAME, SOURCE_NAME) on existing update VALUES (CURRENT TIME,'vrts_host',@objId,'FID_POOL_FT_ERROR', 'vrts_host',@objId, @name, @name)
    end
    
    
    IF (@clusterHostCount != @clusterHostCountBE )
    begin
      insert into P_FAULT_OBJ (TIME_RECEIVED,OBJ_TYPE,OBJ_ID,FID,SOURCE_TYPE,SOURCE_ID, OBJ_NAME, SOURCE_NAME, ARG_LIST) on existing update VALUES (CURRENT TIME,'vrts_cluster',@hcId,'FID_POOL_CLUS_HOST_MISSING_ERROR', 'vrts_cluster',@hcId, @clusterName, @clusterName,@clusterName)
    end

	fetch next c1 into @hcId,@beId, @objId, @name, @ft, @memoryPerServer, @memory, @cps, @cpu,@clusterHostCount,@clusterHostCountBE,@clusterName

	end
close c1
deallocate cursor c1;





if exists ( select 1 from sysevent, sysusers where sysevent.event_name = 'GENERATE_CAPACITY_FAULTS' and 
						   sysevent.creator = sysusers.uid  and 
						   sysusers.name = 'HABDBSYNC' ) then
    drop EVENT GENERATE_CAPACITY_FAULTS
end if;

create event GENERATE_CAPACITY_FAULTS
schedule
        start time '00:00:00' every 3 minutes
        on ( 'Monday', 'Tuesday', 'Wednesday', 'Thursday',
                'Friday','Saturday', 'Sunday' )
handler
begin
        declare @value           varchar(128);
        SELECT value into @value  from habdbsync.P_db_default where name = 'debug_level';
        create variable @sfm_debug_level int;
        SET @sfm_debug_level = cast(@value as integer);
        call habdbsync.SP_create_capacity_faults;
		INSERT INTO SRVCAP_RESERVED_VMS(VM_NAME, HC_ID, CPUS, MEMORY) ON EXISTING UPDATE 
                    (SELECT STORAGE_ID AS VM_NAME, HC_ID AS HC_ID, VM_VPROCESSORS AS CPUS, VM_ALLOCATED_MEM AS MEMORY  FROM  SRVCAP_VIEW_ONLINE_LDOMS);

end;


commit;