#!/opt/VRTSsfmh/bin/perl

#
#$Id: setup.pl,v 1.4 2011/04/07 19:40:27 bdighe Exp $
#$Copyright: Copyright (c) 2011 Symantec Corporation.
#All rights reserved.
#
#THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
#SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
#WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
#The Licensed Software and Documentation are deemed to be commercial
#computer software as defined in FAR 12.212 and subject to restricted
#rights as defined in FAR Section 52.227-19 "Commercial Computer
#Software - Restricted Rights" and DFARS 227.7202, "Rights in
#Commercial Computer Software or Commercial Computer Software
#Documentation", as applicable, and any successor regulations. Any use,
#modification, reproduction release, performance, display or disclosure
#of the Licensed Software and Documentation by the U.S. Government
#shall be solely in accordance with the terms of this Agreement.  $
#


BEGIN { @INC = ("/opt/VRTSsfmh/lib/modules") if ( $^O !~ /Win32/i ) }

use VRTS::AddOnInfo;
use VRTS::Util;
use VRTS::OS;
use File::Copy;
use Archive::Tar;
use VRTS::CMD;
use VRTS::Config;

require File::Path;
require File::Spec;

	my $version = "4.0.523.0";
	my $name = "VRTSVmCapMgmt";
	my $addon_dir = VRTS::Paths::get_path("DeployStoreDir");

	if ($^O =~ /Win32/i) {
		$addon_dir = $addon_dir."\\" . $name ."-".$version;
	} else {
		$addon_dir = $addon_dir.$name."-".$version;
	}

	my $install_dir = VRTS::Paths::get_path("InstallDir");
	my $mhinstall_dir = File::Spec->canonpath($install_dir);
	my $csinstall_dir = $mhinstall_dir;
	$csinstall_dir =~ s/VRTSsfmh/VRTSsfmcs/;

	my $cms_version = &VRTS::Config::MANIFEST_VERSION;
	chomp $cms_version;
	my @build_numbers = split(/\./, $cms_version);
	$cms_build = $build_numbers[2];
	my $supported_build = "400"; #Supports CMS build 400 and above (3.1)

	if ($cms_build < $supported_build) {
		print STDERR "This add-on requires VOM 3.1 or higher.\n";
		print STDERR "Current Version:$cms_version\n";
		exit(1);
	}

	my $EX_PACKAGE = 1;
	my $EX_DB_TABLES = 2;

	#copy the precache file
	my $XML = File::Spec->catfile($addon_dir, "cms", "SrvCapacityPlannerCache.xml");
	my $destdir;
	
	
	
	$destdir = File::Spec->catfile($csinstall_dir, "esmweb", "precache", "web");
	copy($XML, $destdir);
	print("Copying $XML to $destdir\n");
	
	my $smlibdir = File::Spec->catfile($csinstall_dir, "esmweb", "webgui", "sm", "WEB-INF", "lib");
	my $JAR1 = File::Spec->catfile($addon_dir, "cms", "SrvCapacityPlanner.jar");
	copy($JAR1, $smlibdir);
	print("Copying $JAR1 to $smlibdir\n");
	
	my $tomcatlibdir = File::Spec->catfile($csinstall_dir, "esmweb", "tomcat", "lib");
	my $MOCK = File::Spec->catfile($addon_dir, "cms", "mockpools.json");
	copy($MOCK, $tomcatlibdir);
	print("Copying $MOCK to $tomcatlibdir\n");

	$destdir = File::Spec->catfile($csinstall_dir, "esmweb", "webgui", "sm", "plugins","sfm");
	my $TAR = File::Spec->catfile($addon_dir, "cms", "SrvCapacityPlanner.jsp.tar");

	chdir($destdir);
	print "Extracting $TAR at $destdir \n"; 
	my $betar = Archive::Tar->new();
	my @filelist = $betar->read($TAR, 0, { extract=>1 } );
	
	# Extract the help tar file for olh.
    my $olhfilename = File::Spec->catfile($addon_dir, "cms", "VOMLDOMCAPCURRENT_EN_US.tar");
    my $olhtar = Archive::Tar->new();
    $olhdir = File::Spec->catfile($csinstall_dir,"esmweb", "webgui","symhelp","Content");
	print("Extracting help files to $olhdir\n");
    chdir($olhdir);
    my @olhfilelist = $olhtar->read($olhfilename, 0, { extract=>1 } );
	
	
	if (VRTS::AddOnInfo::is_CS()) {
		my $dbisql = File::Spec->catfile($csinstall_dir, "config", "adm", "db_isql.sh");
		my $xdbadm = File::Spec->catfile($mhinstall_dir, "bin", "xdbadm");
		my $log_dir_cs = VRTS::Paths::get_path("AppDir");
		$log_dir_cs =~ s/VRTSsfmh/VRTSsfmcs/g;
		my $odbc_ini_dir = "/etc/vx/VRTSsfmcs";
		$odbc_ini_dir = $log_dir_cs if ( $^O =~ /Win32/i);
		my $cmd;

		if ($^O =~ /Win32/i) {
			$cmd = make_command($xdbadm, "-f", File::Spec->catfile($addon_dir, "cms", "SrvCapacityPlanner.sql"),
									  "-o", File::Spec->catfile($odbc_ini_dir,".odbc.ini"),
									  "-c", File::Spec->catfile($log_dir_cs, "conf"),
									  "-d", "SFMdb3");
			

			


		} else {
			if (!defined $ENV{'SFMDBPASSWD'}) {
				open(HANDLE, $xdbadm." -g -u HABDBSYNC -c ".File::Spec->catfile($log_dir_cs, "conf")." -o ".File::Spec->catfile($odbc_ini_dir, ".odbc.ini"." |")) or $execute_error = 1;
				$pwd = <HANDLE>;
				chomp($pwd);
				close(HANDLE);
				$ENV{'SFMDBPASSWD'} = $pwd;
			}
			
			#Create DB tables
			
			$cmd = make_command($dbisql, "HABDBSYNC", File::Spec->catfile($addon_dir, "cms", "SrvCapacityPlanner.sql"));
		}
		print "Executing \"$cmd\"\n"; 					
		my $out = system($cmd);
		my $ret = $? >> 8;
		if ($ret) {
			print "Addon Creation Failed. error $out. rc $ret";
			exit($ret); 	
		}

	}	
	
	
	
exit 0;
